#!/bin/bash

# Program execution:
# ./scanpdf2djvu.sh scan.pdf [first_page_number]

# Purpose:
# Convert a PDF file of a scanned journal articles to a DjVu file with
# pdf2djvu, using MuPDF to extract image information (DPI and monochrome).

# Plan:
# A PDF file contains content streams commands, which determine how to draw
# an image. We need the transformation matrices used by the command "cm" and
# the image size to calculate the dots per inch. All information we need is
# provided by mupdftool as an XML tree, where the matrices are attributes of
# ancestors of the image element (which has size and color information as
# attributes). To extract the information from the XML output of mupdftool
# the following XSLT stylesheet is employed:
#
# <?xml version="1.0" encoding="utf-8"?>
# <xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
# <xsl:output omit-xml-declaration="yes" indent="no"/>
# <xsl:template match="/">
#   <xsl:for-each select="//image">
#     <xsl:for-each select="ancestor::transform">
#       <xsl:value-of select="@matrix"/>
#       <xsl:value-of select="'&#10;'"/>
#     </xsl:for-each>
#     <xsl:value-of select="'image&#10;'"/>
#     <xsl:value-of select="@w"/>
#     <xsl:value-of select="' '"/>
#     <xsl:value-of select="@h"/>
#     <xsl:value-of select="' '"/>
#     <xsl:value-of select="@n"/>
#     <xsl:if test="not(position()=last())">
#       <xsl:value-of select="'&#10;'"/>
#     </xsl:if>
#   </xsl:for-each>
# </xsl:template>
# </xsl:stylesheet>

# Prerequisites:
# coreutils, MuPDF, grep, gawk, sed, libxslt, pdf2djvu
# get MuPDF with:
# darcs get --lazy http://darcs.ghostscript.com/fitz/
# To update the repository change to the directory 'fitz' and execute 'darcs pull'

# Producer:
# Theo Wollenleben (alpha0x89 yahoo de)
# 2009-05-17

PDF="$1"
if [ -n "$2" ]; then
 SHIFT=$(($2-1))
else
 SHIFT=0
fi
XSL="./scanpdf2djvu.xsl"

if [ -z "$PDF" ]; then
 echo "$0: usage: $0 file.pdf"
 exit
fi
if [ ! -e "$XSL" ]; then
 echo "$0: File not found: $XSL"
 exit
fi
DJVU="$(basename "$PDF" .pdf).djvu"
if [ -e "$DJVU" ]; then
 echo "$0: File exists: $DJVU"
 exit
fi
echo "$PDF"

# mupdftool (2009-02-21) generates invalid XML;
# """, "<", ">", "&" must be replaced by markup "&quot;", "&lt;", "&gt;", "&amp;"
# assume all tranformation matrices (content streams command cm) to be diagonal
PAGES=$(mupdftool info "$PDF"|grep "^Pages"|gawk '{print $2}')
(echo "<mupdftool>"
 mupdftool draw -x "$PDF" 2>/dev/null | tail -n +2
 echo "</mupdftool>") |
sed 's/"""/"\&quot;"/g;s/"</"\&lt;/g;s/>"/\&gt;"/g;s/"&"/"\&amp;"/g' |
xsltproc "$XSL" - |
gawk 'BEGIN{scale_x=1; scale_y=1}
{if($0=="image"){
  getline
  printf "%.0f %.0f %d\n", $1*72/scale_x, $2*72/scale_y, $3
  scale_x=1; scale_y=1}
 else {scale_x*=$1; scale_y*=$4}}' |
uniq -c | (
# need a subshell (or else variables set inside "while read; do ...; done" are lost)
i=1
while read LINE; do
 IMAGES[$i]="$LINE"; ((i++))
done
COUNTS=${#IMAGES[*]}
if [ $COUNTS -eq 0 ]; then
 echo "$0: No image found."
 exit
else
 echo "Images (count x-dpi y-dpi colors):"
 for IMG in "${IMAGES[@]}"; do
  echo $IMG
 done
fi
if [ $COUNTS -gt 1 ]; then
 # sometimes a non-scanned cover page is prepended:
 echo "Images of different resolution or color. Omitting first (cover?) page."
 START=2
else
 START=1
fi
IMG_INFO=(${IMAGES[$COUNTS]})
DPI=${IMG_INFO[1]}
echo "Using --dpi=$DPI"
MONO=""
if [ ${IMG_INFO[3]} -eq 1 ]; then
 MONO="--monochrome"
 echo "Using --monochrome"
fi
pdf2djvu --output="$DJVU" --pageid-template="{dpage+$SHIFT:0*}.djvu" --page-title-template="{dpage+$SHIFT:0*}" --dpi=$DPI $MONO --pages=$START-$PAGES "$PDF"
)
